#include "WebSocketProtocol.h"
#include <iostream>
#include <sstream>
#include <string.h>
#include <arpa/inet.h>
#include "sha1.h"
#include "base64.h"

const char * MAGIC_KEY = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11";

CWebSocketProtocol::CGrabo CWebSocketProtocol::m_grabo;
CWebSocketProtocol * CWebSocketProtocol::m_inst = 0;

CWebSocketProtocol::CWebSocketProtocol()
{
}


CWebSocketProtocol::~CWebSocketProtocol()
{
}



CWebSocketProtocol * CWebSocketProtocol::getInstance()
{
	if (m_inst != 0)
	{
		m_inst = new CWebSocketProtocol;
	}

	return m_inst;
}

int CWebSocketProtocol::getResponseHttp(string &request, string &response)
{
	// httpͷϢ  
	int ret = WS_STATUS_UNCONNECT;
	std::istringstream stream(request.c_str());
	std::string reqType;
	std::getline(stream, reqType);
	if (reqType.substr(0, 4) != "GET ")
	{
		return ret;
	}

	std::string header;
	std::string::size_type pos = 0;
	std::string websocketKey;
	while (std::getline(stream, header) && header != "\r")
	{
		header.erase(header.end() - 1);
		pos = header.find(": ", 0);
		if (pos != std::string::npos)
		{
			std::string key = header.substr(0, pos);
			std::string value = header.substr(pos + 2);
			if (key == "Sec-WebSocket-Key")
			{
				ret = WS_STATUS_CONNECT;
				websocketKey = value;
				break;
			}
		}
	}

	if (ret != WS_STATUS_CONNECT)
	{
		return ret;
	}

	// httpӦͷϢ  
	response = "HTTP/1.1 101 Switching Protocols\r\n";
	response += "Connection: upgrade\r\n";
	response += "Sec-WebSocket-Accept: ";

	std::string serverKey = websocketKey + MAGIC_KEY;

	SHA1 sha;
	unsigned int message_digest[5];
	sha.Reset();
	sha << serverKey.c_str();

	sha.Result(message_digest);
	for (int i = 0; i < 5; i++) {
		message_digest[i] = htonl(message_digest[i]);
	}
	serverKey = base64_encode(reinterpret_cast<const unsigned char*>(message_digest), 20);
	response += serverKey;
	response += "\r\n";
	response += "Upgrade: websocket\r\n\r\n";

	return ret;
}

int CWebSocketProtocol::wsDecodeFrame(string inFrame, string &outMessage)
{
	int ret = WS_OPENING_FRAME;
	const char *frameData = inFrame.c_str();
	const int frameLength = inFrame.size();
	if (frameLength < 2)
	{
		ret = WS_ERROR_FRAME;
	}

	// չλ  
	if ((frameData[0] & 0x70) != 0x0)
	{
		ret = WS_ERROR_FRAME;
	}

	// finλ: Ϊ1ʾѽ, Ϊ0ʾ  
	ret = (frameData[0] & 0x80);
	if ((frameData[0] & 0x80) != 0x80)
	{
		ret = WS_ERROR_FRAME;
	}

	// maskλ, Ϊ1ʾݱ  
	if ((frameData[1] & 0x80) != 0x80)
	{
		ret = WS_ERROR_FRAME;
	}

	//   
	uint16_t payloadLength = 0;
	uint8_t payloadFieldExtraBytes = 0;
	uint8_t opcode = static_cast<uint8_t>(frameData[0] & 0x0f);
	if (opcode == WS_TEXT_FRAME)
	{
		// utf-8ı֡  
		payloadLength = static_cast<uint16_t>(frameData[1] & 0x7f);
		if (payloadLength == 0x7e)
		{
			uint16_t payloadLength16b = 0;
			payloadFieldExtraBytes = 2;
			memcpy(&payloadLength16b, &frameData[2], payloadFieldExtraBytes);
			payloadLength = ntohs(payloadLength16b);
		}
		else if (payloadLength == 0x7f)
		{
			// ݹ,ݲ֧  
			ret = WS_ERROR_FRAME;
		}
	}
	else if (opcode == WS_BINARY_FRAME || opcode == WS_PING_FRAME || opcode == WS_PONG_FRAME)
	{
		// /ping/pong֡ݲ  
	}
	else if (opcode == WS_CLOSING_FRAME)
	{
		ret = WS_CLOSING_FRAME;
	}
	else
	{
		ret = WS_ERROR_FRAME;
	}

	// ݽ  
	if ((ret != WS_ERROR_FRAME) && (payloadLength > 0))
	{
		// header: 2ֽ, masking key: 4ֽ  
		const char *maskingKey = &frameData[2 + payloadFieldExtraBytes];
		char *payloadData = new char[payloadLength + 1];
		memset(payloadData, 0, payloadLength + 1);
		memcpy(payloadData, &frameData[2 + payloadFieldExtraBytes + 4], payloadLength);
		for (int i = 0; i < payloadLength; i++)
		{
			payloadData[i] = payloadData[i] ^ maskingKey[i % 4];
		}

		outMessage = payloadData;
		delete[] payloadData;
	}

	return ret;
}

int CWebSocketProtocol::wsEncodeFrame(string inMessage, string &outFrame, enum WS_FrameType frameType)
{
	int ret = WS_EMPTY_FRAME;
	const uint32_t messageLength = inMessage.size();
	if (messageLength > 32767)
	{
		// ݲ֧ô
		std::cout << "ݲ֧ô" << std::endl;

		return WS_ERROR_FRAME;
	}

	uint8_t payloadFieldExtraBytes = (messageLength <= 0x7d) ? 0 : 2;
	// header: 2ֽ, maskλΪ0(), masking keyд, ʡ4ֽ  
	uint8_t frameHeaderSize = 2 + payloadFieldExtraBytes;
	uint8_t *frameHeader = new uint8_t[frameHeaderSize];
	memset(frameHeader, 0, frameHeaderSize);
	// finλΪ1, չλΪ0, λΪframeType  
	frameHeader[0] = static_cast<uint8_t>(0x80 | frameType);

	// ݳ  
	if (messageLength <= 0x7d)
	{
		frameHeader[1] = static_cast<uint8_t>(messageLength);
	}
	else
	{
		frameHeader[1] = 0x7e;
		uint16_t len = htons(messageLength);
		memcpy(&frameHeader[2], &len, payloadFieldExtraBytes);
	}

	//   
	uint32_t frameSize = frameHeaderSize + messageLength;
	char *frame = new char[frameSize + 1];
	memcpy(frame, frameHeader, frameHeaderSize);
	memcpy(frame + frameHeaderSize, inMessage.c_str(), messageLength);
	frame[frameSize] = '\0';
	outFrame = frame;

	delete[] frame;
	delete[] frameHeader;
	return ret;
}
